!=======================================================================

SUBROUTINE  TIME_LOOP

  USE CONTROL_DATA, ONLY:                                              &
                          MZ, MT1, MT2, LS, KEY_TLV, KEY_TLD,          &
                          KEY_SNV, KEY_SND, IPAS1, IPAS2, MR
  USE GRID_MEDIUM , ONLY:                                              &
                          H, DEN
  USE AUXIL       , ONLY:                                              &
                          SOURTF, NSTFS
  USE WAVEFIELD   , ONLY:                                              &
                          V, SEIS, VELO
  USE TIME        , ONLY:                                              &
                          time_begin, time_end, DATE_TIME, BIG_BEN

!-----------------------------------------------------------------------

  IMPLICIT NONE

  LOGICAL :: EX, RECALL1V, RECALL2V, RECALL1D, RECALL2D
  INTEGER :: ITILMOD1, ITILMOD2, L, ITILE

!-----------------------------------------------------------------------

  INTERFACE

    SUBROUTINE     ANEL_FUNC
    END SUBROUTINE ANEL_FUNC
    
    SUBROUTINE     DISPLACEMENT
    END SUBROUTINE DISPLACEMENT
    
    SUBROUTINE     REC_DIS  (T)
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: T
    END SUBROUTINE REC_DIS
    
    SUBROUTINE     REC_VEL  (T)
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: T
    END SUBROUTINE REC_VEL
    
    SUBROUTINE     SNAP_DIS (T)
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: T
    END SUBROUTINE SNAP_DIS
    
    SUBROUTINE     SNAP_VEL (T)
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: T
    END SUBROUTINE SNAP_VEL
    
    SUBROUTINE     STO_BOR
    END SUBROUTINE STO_BOR
    
    SUBROUTINE     STRAIN
    END SUBROUTINE STRAIN

    SUBROUTINE     STRESS
    END SUBROUTINE STRESS

    SUBROUTINE     VELOCITY
    END SUBROUTINE VELOCITY

  END INTERFACE   
  
!-----------------------------------------------------------------------

  CALL CPU_TIME ( time_begin )

  DO ITILE = MT1, MT2
  
    CALL CPU_TIME ( time_end )
    CALL DATE_AND_TIME ( BIG_BEN(1),BIG_BEN(2),BIG_BEN(3), DATE_TIME )
    PRINT     *, 'CPU time: ', time_end - time_begin, ' seconds'
    WRITE (*,'(''Current time: '',I2,'':'',I2,'':'',F6.3)')              &
    DATE_TIME(5), DATE_TIME(6), DATE_TIME(7)+REAL(DATE_TIME(8)/1000.)
    PRINT     *, 'ITILE=', ITILE
    WRITE (11,'(I6,1X,F10.2,1X,I4,I3,I3,I5,3I3,I4)')                     &
               ITILE, time_end - time_begin, DATE_TIME

    CALL  STO_BOR
    CALL  STRAIN
    CALL  ANEL_FUNC
    CALL  STRESS
    CALL  VELOCITY

    IF ( ITILE <= NSTFS ) THEN
      V (LS) = V (LS) + H*DEN(LS)*SOURTF(ITILE)
    END IF

    CALL  DISPLACEMENT
    
    ITILMOD1 = MOD (ITILE,IPAS1)
    ITILMOD2 = MOD (ITILE,IPAS2)
    RECALL1V = KEY_TLV .AND. (ITILMOD1 == 0)
    RECALL2V = KEY_SNV .AND. (ITILMOD2 == 0)
    RECALL1D = KEY_TLD .AND. (ITILMOD1 == 0)
    RECALL2D = KEY_SND .AND. (ITILMOD2 == 0)

    IF ( RECALL1V ) CALL REC_VEL (ITILE)
    IF ( RECALL1D ) CALL REC_DIS (ITILE)
    IF ( RECALL2V ) CALL SNAP_VEL(ITILE)
    IF ( RECALL2D ) CALL SNAP_DIS(ITILE)

  END DO
 
  IF ( KEY_SND ) THEN
    DO L = 1, MR
      DO ITILE = 1, (MT2-MT1+1)/IPAS2
        WRITE ( 23, "(G10.4,2H, )" ) SEIS(L,ITILE)
      END DO
    END DO
    WRITE(23,*) "]"
    WRITE(23,*) "}"
    CLOSE(23)
  END IF

  IF ( KEY_SNV ) THEN
    DO L = 1, MR
      DO ITILE = 1, (MT2-MT1+1)/IPAS2
        WRITE ( 24, "(G10.4,2H, )" ) VELO(L,ITILE)
      END DO
    END DO
    WRITE(24,*) "]"
    WRITE(24,*) "}"
    CLOSE(24)
  END IF


END  SUBROUTINE  TIME_LOOP
