MODULE MOD_FUNC

! module contains functions for P and S-wave velocities, density and
! quality factors in sediments (1st layer)

IMPLICIT NONE

CONTAINS
!==================== 1D INHOMOGENEOUS MEDIUM =========================
!------------------------------------------------- P or S-wave velocity
  FUNCTION FUNCV (Z)

    USE NRTYPE, ONLY: WP

    REAL(WP), INTENT(IN) :: Z
    REAL(WP)             :: FUNCV
    
    if (z<200.0) then
      !layer
      funcv = 625.
    else
      !halfspace
      funcv = 3126.
    end if

  END FUNCTION FUNCV

!--------------------------------------------------------------- Density
  FUNCTION FUNCRHO (Z)

    USE NRTYPE, ONLY: WP

    REAL(WP), INTENT(IN) :: Z
    REAL(WP)             :: FUNCRHO

    if (z<200.0) then
      !layer
      funcrho = 1600.
    else
      !halfspace
      funcrho = 1800.
    end if

  END FUNCTION FUNCRHO

!------------------------------------------- P or S-wave quality factor
  FUNCTION FUNCQ (Z)

    USE NRTYPE, ONLY: WP

    REAL(WP), INTENT(IN) :: Z
    REAL(WP)             :: FUNCQ
  
    if (z<200.0) then
      !layer
      funcq = 50.
    else
      !halfspace
      funcq = 10000.
    end if

  END FUNCTION FUNCQ


END MODULE
